<?php
// 20141020 - v. 1

class cube {
	function __construct($cubeAddress, $cubePort, $rootName) {
		//saving useful parameter to class var
		$this->rootName = $rootName;

		//managing connection
		$this->fp = @fsockopen($cubeAddress, $cubePort, $errno, $errstr, 5);
		if (!$this->fp){
			if ($errno == 111) throw new Exception("Local MAX! software is running - ".$errstr." - Errno: ".$errno);
			elseif ($errno == 113) throw new Exception("No connection to Cube - ".$errstr." - Errno: ".$errno);
			else throw new Exception("Cube connection error - ".$errstr." - Errno: ".$errno);
		} else {
			$finished = 0;
			$start = time();
			$this->response = "";
			while (!feof($this->fp) && time() < $start+30 && $finished == 0){
				$line = fgets($this->fp);
				if (strpos($line,"L:") !== false) $finished = 1;
				if ($line != "")  $this->response .= $line."\n";
			}
			
			socket_set_blocking($this->fp,true);
			stream_set_timeout($this->fp, 2);
			if ($finished != 1) throw new Exception("No connection to Cube - ".$errstr." - Errno: ".$errno);
		}		
	}
   
	function __destruct() {
		//closing connection
		fclose($this->fp);
	}
	
	//this encodes and decodes datetime in "cube format"
	function formatDateTime($string,$method){
		//in "strings" accepts 1) timestamp (for encoding) 2) 3bytes in hex (for decode)
		//in "method" accepts 1) "enc" for encoding 2) "dec" for decoding (could work on "string" lenght, but better expliciting it..
		
		//returns 1) an array with "date" (2 bytes) and "time" (1 byte) entries ready to be used in commands (for encoding) or 2) a string (timestamp)
		
		switch($method){
			case "enc":

				//working on DATE
				//fetching day number from timestamp
				$day = date("j", $string);
				//now need to convert it to binary
				$day = decbin($day);
				//now need to pad it to 5 digits
				$day = str_pad($day, 5, "0", STR_PAD_LEFT);

				//fetching month number from timestamp
				$month = date("n", $string);
				//now need to convert it to binary
				$month = decbin($month);
				//now need to pad it to 4 digits
				$month = str_pad($month, 4, "0", STR_PAD_LEFT);

				//fetching year number from timestamp
				$year = date("y", $string);
				//now need to convert it to binary
				$year = decbin($year);
				//now need to pad it to 6 digits
				$year = str_pad($year, 6, "0", STR_PAD_LEFT);

				//now building two bytes
				$bytes = substr($month,0,3).$day.substr($month,3,1)."0".$year;

				//now converting to hex!
				$dateConv = dechex(bindec($bytes));

				//working on TIME
				$hour = date("G", $string);
				$min = date("i", $string);
				//flooring minutes to 0 or 0,5
				if($min<30) $min = ",0";
				else $min = ",5";

				$time = $hour.$min;
				//*2 for encoding, letting PHP casting the string to number
				$time = $time*2;
				//converting to hex and i'm over
				$time = dechex($time);
				
				return array("date" => $dateConv, "time" => $time);
			
			break;
			case "dec":
				//first two bytes are date until. Fetching one by one and converting to bin string..
				$dateUntilOne = str_pad(decbin(hexdec(bin2hex(substr($string,0,1)))),8,"0",STR_PAD_LEFT);
				$dateUntilTwo = str_pad(decbin(hexdec(bin2hex(substr($string,1,1)))),8,"0",STR_PAD_LEFT);
				//concatenating to have a 16bit string
				$dateUntil = $dateUntilOne.$dateUntilTwo; 
				//time until is the third byte..
				$timeUntil = (hexdec(bin2hex(substr($string,2,1)))*0.5);
				$yearUntil = substr($dateUntil,10,6);
				$monthUntil = substr($dateUntil,0,3).substr($dateUntil,8,1);
				$dayUntil = substr($dateUntil,3,5);
				$timestampUntil = mktime(floor($timeUntil),($timeUntil-floor($timeUntil))*60,0,bindec($monthUntil),bindec($dayUntil),bindec($yearUntil));								
				return $timestampUntil;		
			break;
		}
	}

	//this does the dirty job: parse all the cube response and returns a complex array
	function parseCube(){
		$cubeOut = explode("\n\n",substr($this->response,0,-2));
		//this array will keep everything
		$rooms = Array();
		//this is only temporary
		$config = Array();
		//fetching every single line..
		foreach($cubeOut as $singleOut){
			//..and switching on response type
			switch(substr($singleOut,0,2)){
				case "H:":
					//this is hello entry. It contains some cube info. Not base64 encoded, they are just CSV. So it's enough to explode the string and read entries..
					$lineH = substr($singleOut,2);
					$hPieces = explode(",",$lineH);
					
					//manually building room with id "0" to allocate eco button and cube info. Trying to keep the same structure than other rooms, but there will not be a "heating" node here
					$rooms[0]['Name'] = $this->rootName;
					$rooms[0]['Devices'][0][strtoupper($hPieces[1])]["Name"] = "Max! Cube";
					$rooms[0]['Devices'][0][strtoupper($hPieces[1])]["Serial"] = strtoupper($hPieces[0]);
					$rooms[0]['Devices'][0][strtoupper($hPieces[1])]["Firmware"] = hexdec($hPieces[2]);
					$rooms[0]['Devices'][0][strtoupper($hPieces[1])]['Info']["DutyCycle"] = hexdec($hPieces[5]);	
					
					//Quite useless informations.. cube date, time, connection info and some unknown..
					//$rooms[0]['Devices'][0][strtoupper($hPieces[1])]["UnknownField1"] = $hPieces[3]; //this seems to be always 00000000
					$rooms[0]['Devices'][0][strtoupper($hPieces[1])]['Info']["ConnectionID"] = $hPieces[4];
					//$rooms[0]['Devices'][0][strtoupper($hPieces[1])]["UnknownField2"] = $hPieces[6]; //this seems to be always 32
					$rooms[0]['Devices'][0][strtoupper($hPieces[1])]['Info']["CubeTimestamp"] = mktime(hexdec(substr($hPieces[8],0,2)),hexdec(substr($hPieces[8],2,2)),0,hexdec(substr($hPieces[7],2,2)),hexdec(substr($hPieces[7],4,2)),hexdec(substr($hPieces[7],0,2)));
					//$rooms[0]['Devices'][0][strtoupper($hPieces[1])]["UnknownField3"] = $hPieces[9]; //?
					//$rooms[0]['Devices'][0][strtoupper($hPieces[1])]["UnknownField4"] = $hPieces[10]; //?
					
					//H entry is over.					
					break;
				case "M:":
					/*
					This is Metadata entry. It contains informations regarding rooms and devices. It's a CSV string where first and second pieces are unknown, while the third is base64 encoded and has this structure:
					first 2 bytes: Unknown(maybe METADATA-VERSION)
					next 1 byte: number of rooms

					followed by number of rooms times:
					next 1 byte: roomID
					next 1 byte: length of roomname
					next (length of roomname) bytes: roomname
					next 3 bytes: rf-adress of first room device (some kind of useless)

					followed by:
					next 1 byte: number of devices

					followed by number of devices times:
					next 1 byte: deviceType (1:thermostat,4:window-contact,5:eco-switch)
					next 3 bytes: rf-adress of device 
					next 10 bytes: serial
					next 1 byte: length of devicename
					next (length of devicename) bytes: devicename
					next 1 byte: roomid
					*/
					$decodedM = base64_decode(substr($singleOut,8)); //offset 8 to jump over "M:00,01," string
					$index = 2; //jumping above 2 unknows bytes
					//looking for rooms: reading room number, looping in data "room number" times. Building array for device content.
					$roomNumber = hexdec(bin2hex(substr($decodedM,$index,1)));
					$index ++;
					for($r = 0; $r < $roomNumber; $r++){
						//reading rooms data
						$roomID = hexdec(bin2hex(substr($decodedM,$index,1)));
						$index++;
						$roomNameLength = hexdec(bin2hex(substr($decodedM,$index,1)));
						$index++;
						$roomName = substr($decodedM,$index,$roomNameLength);
						$index += $roomNameLength;
						$index += 3; //useless data: rf address of first room device
						$rooms[$roomID]['Name'] = $roomName;
					}
					//after rooms, looking for devices: same loop as for rooms, but inserting useful data in room array
					$deviceNumber = hexdec(bin2hex(substr($decodedM,$index,1)));
					$index ++;
					$devices = Array();
					for($d = 0; $d < $deviceNumber; $d++){
						//reading devices data
						$deviceType = hexdec(bin2hex(substr($decodedM,$index,1)));
						$index++;
						$deviceAddress = strtoupper(bin2hex(substr($decodedM,$index,3)));
						$index += 3;
						$deviceSerial = strtoupper(substr($decodedM,$index,10));
						$index += 10;
						$deviceNameLength = hexdec(bin2hex(substr($decodedM,$index,1)));
						$index++;
						$deviceName = substr($decodedM,$index,$deviceNameLength);
						$index += $deviceNameLength;
						$roomID = hexdec(bin2hex(substr($decodedM,$index,1)));
						$index += 1;
						$rooms[$roomID]['Devices'][$deviceType][$deviceAddress]["Serial"] = $deviceSerial;
						$rooms[$roomID]['Devices'][$deviceType][$deviceAddress]["Name"] = $deviceName;
						$devices[$deviceAddress]["Type"] = $deviceType;
						$devices[$deviceAddress]["Room"] = $roomID;
					}
					//M entry is over.					
					
					break;
				case "C:":
					//this is configuration entry, so let's insert it in a $config array. The array key is the RF Address of the device.
					//L entry comes then and i don't need all the informations. Configuration data will be linked to devices through the RF address only.
					$lineC = substr($singleOut,2);
					$cPieces = explode(",",$lineC);
					$decodedC = base64_decode($cPieces[1]);
					//first byte is data lenght. Quite useless here..
					//$cDataLenght = hexdec(bin2hex(substr($decodedC,0,1)));
					//then 3 bytes of rfaddress. Quite useless here again.. i already have it..
					$deviceAddress = strtoupper(bin2hex(substr($decodedC,1,3)));
					//what's next? finally the device type! this will switch how to fetch informations..
					$cDevType = hexdec(bin2hex(substr($decodedC,4,1)));
					//then 2 info bytes: roomID (quite useless..), and firmware version (both are not working for the cube, so filtering them)
					if($cDevType != 0) {
						$cRoomId = hexdec(bin2hex(substr($decodedC,5,1)));
						$cDeviceFirmware = hexdec(bin2hex(substr($decodedC,6,1)));
						//hammering this inside devices info.
						$rooms[$cRoomId]['Devices'][$cDevType][$deviceAddress]["Firmware"] = $cDeviceFirmware;
					}
					//then an unknown byte
					//$cUnknown1 = dechex(ord(substr($decodedC,7,3)));
					//then 10 bytes serial number
					$cDevSerial = htmlentities(substr($decodedC,8,10));
					switch($cDevType){
						case "0":
							//this is the cube itself. Info are not in $config array because cube is already allocated in $rooms[0]
							//1 byte for remote control switch
							$rooms[0]['Devices'][0][$deviceAddress]['Config']['PortalEnabled'] = (dechex(ord(substr($decodedC,18,1))) == 1)?TRUE:FALSE;
							//then a lot of unknown stuff.. 66 bytes
							//$cubeUnknown1 = dechex(ord(substr($decodedC,19,66)));
							//then portal url!
							$rooms[0]['Devices'][0][$deviceAddress]['Config']['PortalUrl'] = substr($decodedC,85,36);
							//then again 118 bytes unknown
							//$cubeUnknown2 = dechex(ord(substr($decodedC,121,118)));
							
						break;
						case "2":
							//this is the radiator thermostat
							$config[$deviceAddress]["ComfortTemperature"] = (ord(substr($decodedC,18,1))/2)."";
							$config[$deviceAddress]["EcoTemperature"] = (ord(substr($decodedC,19,1))/2)."";
							$config[$deviceAddress]["MaxSetPointTemperature"] = (ord(substr($decodedC,20,1))/2)."";
							$config[$deviceAddress]["MinSetPointTemperature"] = (ord(substr($decodedC,21,1))/2)."";
							$config[$deviceAddress]["TemperatureOffset"] = ((ord(substr($decodedC,22,1)))/2-3.5)."";
							$config[$deviceAddress]["WindowOpenTemperature"] = (ord(substr($decodedC,23,1))/2)."";
							$config[$deviceAddress]["WindowOpenDuration"] = dechex(ord(substr($decodedC,24,1)))."";
							$boost = str_pad(decbin(ord(substr($decodedC,25,1))),8,"0",STR_PAD_LEFT)."";
							$config[$deviceAddress]["BoostDuration"] = bindec(substr($boost,0,3))*5;
							$config[$deviceAddress]["BoostValue"] = bindec(substr($boost,3,5))*5;
							$decalc = str_pad(decbin(ord(substr($decodedC,26,1))),8,"0",STR_PAD_LEFT)."";
							$config[$deviceAddress]["DecalcDay"] = bindec(substr($decalc,0,3));
							$config[$deviceAddress]["DecalcTime"] = bindec(substr($decalc,3,5));
							$config[$deviceAddress]["MaximumValveSetting"] = hexdec(bin2hex(substr($decodedC,27,1)))*(100/255)."";
							$config[$deviceAddress]["ValveOffset"] = hexdec(bin2hex(substr($decodedC,28,1)))*(100/255)."";
						break;
						case "3":
							//this is the wall thermostat					
							//only first 4 bytes as the radiator thermostat..
							$config[$deviceAddress]["ComfortTemperature"] = (ord(substr($decodedC,18,1))/2)."";
							$config[$deviceAddress]["EcoTemperature"] = (ord(substr($decodedC,19,1))/2)."";
							$config[$deviceAddress]["MaxSetPointTemperature"] = (ord(substr($decodedC,20,1))/2)."";
							$config[$deviceAddress]["MinSetPointTemperature"] = (ord(substr($decodedC,21,1))/2)."";
							//fetching weekly program data for Wall Thermostat device
							$cIndex = 22;
							for($k=0;$k<(26*7);$k=$k+26){
								//switch to order days as a normal mind..
								switch($k){
									case "0":
										//it's saturday so day 6
										$day = 6;
									break;
									case "26":
										//it's sunday so day 7
										$day = 7;
									break;
									case "52":
										//it's monday so day 1
										$day = 1;
									break;
									case "78":
										//it's Tuesday so day 2
										$day = 2;
									break;
									case "104":
										//it's Wednesday so day 3
										$day = 3;
									break;
									case "130":
										//it's Thursday so day 4
										$day = 4;
									break;
									case "156":
										//it's friday so day 5
										$day = 5;
									break;
									
								}
								$allDayProgram = substr($decodedC,$cIndex+$k,26);
								$walkDay = TRUE; //need this to read only one time the last program..
								for($i=0;$i<26;$i=$i+2){
									if($walkDay === TRUE){
										$singleDayProgram = str_pad(decbin(ord(substr($allDayProgram,$i,1))),8,"0",STR_PAD_LEFT).str_pad(decbin(ord(substr($allDayProgram,$i+1,1))),8,"0",STR_PAD_LEFT);
										$minutes = bindec(substr($singleDayProgram,7,9))*5;
										if($minutes == 1440) $walkDay = FALSE; //1440 minutes are midnight, so the day is over!
										$temp = bindec(substr($singleDayProgram,1,6))/2;
										$timestamp = strtotime(intval($minutes/60).":".str_pad($minutes%60,2,"0",STR_PAD_LEFT));
										$rooms[$devices[$deviceAddress]["Room"]]['Heating']['WeeklyProgram'][$day][] = Array("Temp" => $temp, "Until" => $timestamp);
									}
								}
							}
						break;
						case "5":
							//this is the eco switch
							//seems to be empty.. boh?
						break;
					}	
	
					//putting config information in the rooms array. Not for cube, obviously.. And for eco switch, seen it's empty..
					if($cDevType != 0 && $cDevType != 5) $rooms[$devices[$deviceAddress]["Room"]]['Devices'][$devices[$deviceAddress]["Type"]][$deviceAddress]['Config'] = $config[$deviceAddress];
								
					break;
				case "L:":
					//this is realtime response. This contains the state of every device.
					$decodedL = base64_decode(substr($singleOut,2));
					$index = 0;
					while($index < strlen($decodedL)){
						//first byte is length of data for single device.
						$dataLen = hexdec(bin2hex(substr($decodedL,$index,1)));
						//then there is the device RF addres. It's common in all device types..
						$deviceAddress = strtoupper(bin2hex(substr($decodedL,$index+1,3)));
						//one unknown byte then.. seems always 09
						//$lUnknown1 = bin2hex(substr($decodedL,$index+4,1));
						//the next byte is about some quite useless informations.. Only 4 bits are known
						$infos = str_pad(decbin(ord(substr($decodedL,$index+5,1))),8,"0",STR_PAD_LEFT);
						if(substr($infos,3,1) == 0){
							//Info Validity KO;
							$rooms[$devices[$deviceAddress]["Room"]]['Devices'][$devices[$deviceAddress]["Type"]][$deviceAddress]["Info"]["InfoValidity"] = FALSE;
						} else {
							//Info Validity OK;
							$rooms[$devices[$deviceAddress]["Room"]]['Devices'][$devices[$deviceAddress]["Type"]][$deviceAddress]["Info"]["InfoValidity"] = TRUE;
						}
						if(substr($infos,4,1) == 0){
							//No error;
							$rooms[$devices[$deviceAddress]["Room"]]['Devices'][$devices[$deviceAddress]["Type"]][$deviceAddress]["Info"]["Error"] = FALSE;
						} else {
							//Error occurred;
							$rooms[$devices[$deviceAddress]["Room"]]['Devices'][$devices[$deviceAddress]["Type"]][$deviceAddress]["Info"]["Error"] = TRUE;
						}					
						if(substr($infos,5,1) == 0){
							//In not answer;
							$rooms[$devices[$deviceAddress]["Room"]]['Devices'][$devices[$deviceAddress]["Type"]][$deviceAddress]["Info"]["Answer"] = FALSE;
						} else {
							//Is answer;
							$rooms[$devices[$deviceAddress]["Room"]]['Devices'][$devices[$deviceAddress]["Type"]][$deviceAddress]["Info"]["Answer"] = TRUE;
						}
						if(substr($infos,6,1) == 0){
							//Status not initialized;
							$rooms[$devices[$deviceAddress]["Room"]]['Devices'][$devices[$deviceAddress]["Type"]][$deviceAddress]["Info"]["Initialized"] = FALSE;
						} else {
							//Status initialized;
							$rooms[$devices[$deviceAddress]["Room"]]['Devices'][$devices[$deviceAddress]["Type"]][$deviceAddress]["Info"]["Initialized"] = TRUE;
						}		

						//then status byte. This is much more interesting.
						$status = str_pad(decbin(ord(substr($decodedL,$index+6,1))),8,"0",STR_PAD_LEFT);
						if(substr($status,0,1) == 0){
							//Battery status OK;
							$rooms[$devices[$deviceAddress]["Room"]]['Devices'][$devices[$deviceAddress]["Type"]][$deviceAddress]["Info"]["BatteryLow"] = FALSE;
						} else {
							//Battery status KO;
							$rooms[$devices[$deviceAddress]["Room"]]['Devices'][$devices[$deviceAddress]["Type"]][$deviceAddress]["Info"]["BatteryLow"] = TRUE;
						}
						if(substr($status,1,1) == 0){
							//Link Status OK;
							$rooms[$devices[$deviceAddress]["Room"]]['Devices'][$devices[$deviceAddress]["Type"]][$deviceAddress]["Info"]["LinkOk"] = TRUE;
						} else {
							//Link Status KO;
							$rooms[$devices[$deviceAddress]["Room"]]['Devices'][$devices[$deviceAddress]["Type"]][$deviceAddress]["Info"]["LinkOk"] = FALSE;
						}					
						if(substr($status,2,1) == 0){
							//Panel unlocked;
							$rooms[$devices[$deviceAddress]["Room"]]['Devices'][$devices[$deviceAddress]["Type"]][$deviceAddress]["Info"]["PanelLock"] = FALSE;
						} else {
							//Panel locked;
							$rooms[$devices[$deviceAddress]["Room"]]['Devices'][$devices[$deviceAddress]["Type"]][$deviceAddress]["Info"]["PanelLock"] = TRUE;
						}
						if(substr($status,3,1) == 0){
							//Gateway Unknown;
							$rooms[$devices[$deviceAddress]["Room"]]['Devices'][$devices[$deviceAddress]["Type"]][$deviceAddress]["Info"]["Gateway"] = FALSE;
						} else {
							//Gateway Known;
							$rooms[$devices[$deviceAddress]["Room"]]['Devices'][$devices[$deviceAddress]["Type"]][$deviceAddress]["Info"]["Gateway"] = TRUE;
						}							
						if(substr($status,4,1) == 0){
							//DST inactive;
							$rooms[$devices[$deviceAddress]["Room"]]['Devices'][$devices[$deviceAddress]["Type"]][$deviceAddress]["Info"]["DST"] = FALSE;
						} else {
							//DST active;
							$rooms[$devices[$deviceAddress]["Room"]]['Devices'][$devices[$deviceAddress]["Type"]][$deviceAddress]["Info"]["DST"] = TRUE;
						}							
						//the next bit seems unused. Then there is a couple of bit regarding heating mode, read in the wall thermostat switch.

						//here work on specific data! Discovering device type from data length
						switch($dataLen){
							case "6":
								//6 bytes is the eco switch. There's nothing more for me..
								break;
							case "11":
								//11 bytes is a radiator thermostat (for sure the + version).
								//I can fetch from here the room info "valve aperture percentage"
								//Note that for rooms with more than a radiator thermostat, the value is overwritten supposing aperture % is the same for every device.
								$rooms[$devices[$deviceAddress]["Room"]]['Heating']["Aperture"] = hexdec(bin2hex(substr($decodedL,$index+7,1)));
								break;
							case "12":
								//12 bytes is a wall thermostat
								//I can fetch from here the room info "heating mode", "Temperature Set", "Actual Temperature"
								
								//ok talking about temperatures. 12th byte is actual temp*10, so theres a limit: 25,5 degrees. if it's higher, the count starts from 0 again and the firs bit of byte 8 (set temp) is set to one. have to manage this
								$tooHot = str_pad(decbin(ord(substr($decodedL,$index+8,1))),8,"0",STR_PAD_LEFT);
								if(substr($tooHot,0,1) == 0){
									//this is normale case.
									$rooms[$devices[$deviceAddress]["Room"]]['Heating']["TempSet"] = (hexdec(bin2hex(substr($decodedL,$index+8,1)))/2);
									$rooms[$devices[$deviceAddress]["Room"]]['Heating']["TempNow"] = (hexdec(bin2hex(substr($decodedL,$index+12,1)))/10);
								} else {
									//this is hot case! Can manage this in a lot ow ways, now just adding and subtracting some number :)
									$rooms[$devices[$deviceAddress]["Room"]]['Heating']["TempSet"] = (hexdec(bin2hex(substr($decodedL,$index+8,1)))/2)-64;
									$rooms[$devices[$deviceAddress]["Room"]]['Heating']["TempNow"] = (hexdec(bin2hex(substr($decodedL,$index+12,1)))/10)+25.5;
								}
								//There are many informations linked to the room more than to the single device. I'll fetch them from the wall thermostat. I'm not going to check if they are the same in the radiator thermostat..
								//heating mode
								switch(substr($status,-2)){
									case "00":
										//Mode AUTO
										$rooms[$devices[$deviceAddress]["Room"]]['Heating']["Mode"] = "Auto";
										break;
									case "01":
										//Mode MANUAL
										$rooms[$devices[$deviceAddress]["Room"]]['Heating']["Mode"] = "Manual";
										break;
									case "10":
										//Mode VACATION
										$rooms[$devices[$deviceAddress]["Room"]]['Heating']["Mode"] = "Vacation";
										break;
									case "11":
										//Mode BOOST
										$rooms[$devices[$deviceAddress]["Room"]]['Heating']["Mode"] = "Boost";
										break;
								}
								//in vacation mode, i can also fetch date and time until..
								//using decoding function instead
								$rooms[$devices[$deviceAddress]["Room"]]['Heating']['TimestampUntil'] = $this->formatDateTime(substr($decodedL,$index+9,3), "dec");
								break;
						}
						$index += 1+$dataLen;
					} //closing L entry lenght while
				break;
			}	//closing entry type switch
		} //closing cubeOut foreach
		unset($config);
		return $rooms;
	}
	
	//this sends commands to the cuuuuube
	function sendCommand($command){	
		//sending command
		$result = fwrite($this->fp, $command);
		//checking write
		if(!$result) throw new Exception("Command sending failed - $result false");
		$info = stream_get_meta_data($this->fp);
		if($info['timed_out']) throw new Exception("Command sending failed - timeout");
		//reading answer
		$out = fgets($this->fp);
		//checking command status
		$status = substr($out,5,1); //0 is good, 1 is rejected
		if($status == 1) throw new Exception("Command sending failed - Command rejected.");
		//fetching duty cycle
		$dc = substr($out,2,2);
		//converting to dec
		$dc = hexdec($dc);
		//fetching free memory slots
		$ms = substr($out,7,2);
		//converting to dec
		$ms = hexdec($ms);
		$result = Array();
		$result['dutyCycle'] = $dc;
		$result['memorySlots'] = $ms;
		return $result;
	}
	
	//function to create set command for the cuuuuuuuuuube
	function setCube($mode, $room = "00", $temp = "0", $device = "", $timestamp = ""){
		/*the format of the message will be this:
		00		1		00			?, seems to be always this value
		01		1		04			?, seems to be always this value
		02		1		40			?, seems to be always this value
		03		1		00			?, seems to be always this value
		04		1		00			?, seems to be always this value
		05		1		00			?, seems to be always this value     
		06		3		00FE30		RF address valvue. This is optional, you can set only a room id
		09		1		01			?, seems to roomnumber or nr (first valvue is 01, second is 02)  
		10		1		A8			Temperature setpoint, Temp uses 6 bits LSB6 (bit 2 tm 7), 20 deg C = bits 101000 = dec 40/2 = 20 deg C, you need 8 bits to send so add the 2 bits below (sample 10101000 = hex A8)
										bit 0,1 is mode 
											00 = Auto weekprog (no temp is needed, just make the whole byte 00
											01 = Permanent
											10 = Temporarily 
											11 = Boost
		11		2		8B8B        Date until (11-09-2011) (see Encoding/Decoding date/time)
		13		1		1F          Time until (15:30) (see Encoding/Decoding date/time)
		
		Date encoding:
		868B		1000011010001011
					MMMDDDDDM YYYYYY
					100     1       	1001b = 9d  = month
					   00110        	110b = 6d  = day
							  001011	1011b = 11d = year-2000

		Time encoding:
		04 4 * 0.5 hours = 02:00
		*/
		$message = "000440000000"; //this is static part of the command
		//no control on device passed, just appending it, but padding left with zeroes (to 6)
		$message .= str_pad($device,6,"0",STR_PAD_LEFT);
		//no control on room passed, converting to hex and padding left with zeroes (to 2)
		$message .= str_pad(dechex($room),2,"0",STR_PAD_LEFT);
		//no control on mode passed, just padding left with zeroes (to 2)
		$mode = str_pad($mode,2,"0",STR_PAD_LEFT);
		//now working on temp. Need to double it, convert to binary and pad it to 6.
		$temp = $temp*2;
		$temp = decbin($temp);
		$temp = str_pad($temp, 6, "0", STR_PAD_LEFT);
		//now concatenating mode and temp
		$byte = $mode.$temp;
		//and converting to hex.. need to pad it to two
		$byte = str_pad(dechex(bindec($byte)),2,"0",STR_PAD_LEFT);
		//adding to message
		$message .= $byte;
		//working on timestamp. Needed only if different from 0, else just appending 000000
		if($timestamp != ""){
			$dateTime = $this->formatDateTime($timestamp,"enc");
			//adding date to message
			$message .= $dateTime['date'];
			//adding time to message
			$message .= $dateTime['time'];
		};
		//base64 encoding message
		//$message = base64_encode(hex2bin($message));
		$message = base64_encode(pack("H*", $message));
		//encapsulating message
		$message = "s:".$message."\r\n";
		return $message;
	}
}
?>